//---------------------------------------------------------------------------


#pragma hdrstop

#include "CChartTools.h"
#include <Series.hpp>
#include "Math.hpp"
#include <cmath>
//---------------------------------------------------------------------------

#pragma package(smart_init)

//---------------------------------------------------------------------------
CChartTools::CChartTools()
{
	iDefaultMargLeft = 4;
	iDefaultMargRight = 4;
	iDefaultMargBottom = 4;
	iDefaultMargTop = 4;
	iDefaultLegendMargin = 4;
	iDefaultTitleMargin = 4;
	iDefaultVerticalExtraPos = 12;
	iDefaultHorizontalExtraPos = 12;

	bVerticalAxisInOneStack = false;
	bHorizontalAxisInOneStack = false;
}
//---------------------------------------------------------------------------
int CChartTools::getDefaultMargLeft()
{
	return iDefaultMargLeft;
}
//---------------------------------------------------------------------------
int CChartTools::getDefaultMargRight()
{
	return iDefaultMargRight;
}
//---------------------------------------------------------------------------
int CChartTools::getDefaultMargBottom()
{
	return iDefaultMargBottom;
}
//---------------------------------------------------------------------------
int CChartTools::getDefaultMargTop()
{
	return iDefaultMargTop;
}
//---------------------------------------------------------------------------
bool CChartTools::getVerticalAxisInOneStack()
{
	return bVerticalAxisInOneStack;
}
//---------------------------------------------------------------------------
void CChartTools::setVerticalAxisInOneStack(bool _bVerticalAxisInOneStack)
{
	bVerticalAxisInOneStack = _bVerticalAxisInOneStack;
}
//---------------------------------------------------------------------------
bool CChartTools::getHorizontalAxisInOneStack()
{
	return bHorizontalAxisInOneStack;
}
//---------------------------------------------------------------------------
void CChartTools::setHorizontalAxisInOneStack(bool _bHorizontalAxisInOneStack)
{
	bHorizontalAxisInOneStack = _bHorizontalAxisInOneStack;
}
//---------------------------------------------------------------------------
int CalcVerticalTitleSize(TChartAxisTitle* pTChartAxisTitle)
{
	if(pTChartAxisTitle->Angle == 0 || pTChartAxisTitle->Angle == 180 || pTChartAxisTitle->Angle == 360)
	{
		return pTChartAxisTitle->Width;
	}
	else
	if(pTChartAxisTitle->Angle == 90 || pTChartAxisTitle->Angle == 270)
	{
		return pTChartAxisTitle->Height;
	}
	else
	{
		return Max(pTChartAxisTitle->Width, pTChartAxisTitle->Height);
	}
}
//---------------------------------------------------------------------------
int CChartTools::PlaceLeftVerticalAxesOnStack(TCustomChart* Chart,
	int& NextXLeft, int& NextXRight, int& MargLeft, int& MargRight)
{
	int iCustomVerticalAxesCount = 0;

	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == false &&
			Chart->CustomAxes->Items[i]->OtherSide == false)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomVerticalAxesCount;
			}
		}
	}

	if(IsAxisVisible(Chart, Chart->LeftAxis))
	{
		++iCustomVerticalAxesCount;
	}

	if(iCustomVerticalAxesCount)
	{
		double dSeparation = 1;
		double dStartPosition = 0.0;
		double dStep = 100.0/iCustomVerticalAxesCount;
		double dEndPosition = dStep + dSeparation;

		int iMaxWidth = 0;
		int iAxisWidth = 0;

		//przelicz
		if(IsAxisVisible(Chart, Chart->LeftAxis))
		{
			if(Chart->LeftAxis->PositionUnits != muPercent)
			{
				Chart->LeftAxis->PositionUnits = muPercent;
			}
			Chart->LeftAxis->PositionPercent = 0;
			Chart->LeftAxis->StartPosition = dStartPosition;
			Chart->LeftAxis->EndPosition = dEndPosition - dSeparation;

			dStartPosition = Chart->LeftAxis->EndPosition + dSeparation;
			dEndPosition = dStartPosition + dStep;

			iAxisWidth = CalcVerticalAxisWidth(Chart->LeftAxis);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == false &&
				Chart->CustomAxes->Items[i]->OtherSide == false)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{

					if(Chart->CustomAxes->Items[i]->PositionUnits != muPercent)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPercent;
					}
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = dStartPosition;
					Chart->CustomAxes->Items[i]->EndPosition = dEndPosition - dSeparation;

					dStartPosition = Chart->CustomAxes->Items[i]->EndPosition + dSeparation;
					dEndPosition = dStartPosition + dStep;

					iMaxWidth = Max(iMaxWidth, CalcVerticalAxisWidth(Chart->CustomAxes->Items[i]));
				}
			}
		}
		if(iMaxWidth > 0)
		{
			iMaxWidth = iMaxWidth - iAxisWidth;//Jeeli jest o lewa to wymiar jest ju zmniejszony
		}

		NextXLeft = NextXLeft - iMaxWidth;
		MargLeft = MargLeft + iMaxWidth;
	}

	return iCustomVerticalAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::PlaceRightVerticalAxesOnStack(TCustomChart* Chart,
	int& NextXLeft, int& NextXRight, int& MargLeft, int& MargRight)
{
	int iCustomVerticalAxesCount = 0;

	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == false &&
			Chart->CustomAxes->Items[i]->OtherSide == true)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomVerticalAxesCount;
			}
		}
	}

	if(IsAxisVisible(Chart, Chart->RightAxis))
	{
		++iCustomVerticalAxesCount;
	}

	if(iCustomVerticalAxesCount)
	{
		double dSeparation = 1;
		double dStartPosition = 0.0;
		double dStep = 100.0/iCustomVerticalAxesCount;
		double dEndPosition = dStep + dSeparation;

		int iMaxWidth = 0;
		int iAxisWidth = 0;

		//przelicz
		if(IsAxisVisible(Chart, Chart->RightAxis))
		{
			if(Chart->RightAxis->PositionUnits != muPercent)
			{
				Chart->RightAxis->PositionUnits = muPercent;
			}
			Chart->RightAxis->PositionPercent = 0;
			Chart->RightAxis->StartPosition = dStartPosition;
			Chart->RightAxis->EndPosition = dEndPosition - dSeparation;

			dStartPosition = Chart->RightAxis->EndPosition + dSeparation;
			dEndPosition = dStartPosition + dStep;

			iAxisWidth = CalcVerticalAxisWidth(Chart->RightAxis);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == false &&
				Chart->CustomAxes->Items[i]->OtherSide == true)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{
					if(Chart->CustomAxes->Items[i]->PositionUnits != muPercent)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPercent;
                    }
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = dStartPosition;
					Chart->CustomAxes->Items[i]->EndPosition = dEndPosition - dSeparation;

					dStartPosition = Chart->CustomAxes->Items[i]->EndPosition + dSeparation;
					dEndPosition = dStartPosition + dStep;

					iMaxWidth = Max(iMaxWidth, CalcVerticalAxisWidth(Chart->CustomAxes->Items[i]));
				}
			}
		}
		if(iMaxWidth > 0)
		{
			iMaxWidth = iMaxWidth - iAxisWidth;//Jeeli jest o prawa to wymiar jest ju zmniejszony
		}

		NextXRight = NextXRight - iMaxWidth;
		MargRight = MargRight + iMaxWidth;
	}

	return iCustomVerticalAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::PlaceVerticalAxes(TCustomChart* Chart,
	int& NextXLeft, int& NextXRight, int& MargLeft, int& MargRight)
{
	int iCustomVerticalAxesCount = 0;

	if(bVerticalAxisInOneStack)
	{
		iCustomVerticalAxesCount += PlaceLeftVerticalAxesOnStack(Chart, NextXLeft, NextXRight, MargLeft, MargRight);
		iCustomVerticalAxesCount += PlaceRightVerticalAxesOnStack(Chart, NextXLeft, NextXRight, MargLeft, MargRight);
	}
	else
	{
		if(IsAxisVisible(Chart, Chart->LeftAxis))
		{
			if(Chart->LeftAxis->PositionUnits != muPixels)
			{
				Chart->LeftAxis->PositionUnits = muPixels;
            }
			Chart->LeftAxis->PositionPercent = 0;
			Chart->LeftAxis->StartPosition = 0;
			Chart->LeftAxis->EndPosition = 100;
			StandardVerticalAxes(Chart, Chart->LeftAxis, NextXLeft, NextXRight, MargLeft, MargRight);
		}

		if(IsAxisVisible(Chart, Chart->RightAxis))
		{
			if(Chart->RightAxis->PositionUnits != muPixels)
			{
				Chart->RightAxis->PositionUnits = muPixels;
			}
			Chart->RightAxis->PositionPercent = 0;
			Chart->RightAxis->StartPosition = 0;
			Chart->RightAxis->EndPosition = 100;
			StandardVerticalAxes(Chart, Chart->RightAxis, NextXLeft, NextXRight, MargLeft, MargRight);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == false)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{
					if(Chart->CustomAxes->Items[i]->PositionUnits != muPixels)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPixels;
                    }
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = 0;
					Chart->CustomAxes->Items[i]->EndPosition = 100;
					iCustomVerticalAxesCount += PlaceVerticalAxes(Chart, Chart->CustomAxes->Items[i], NextXLeft, NextXRight, MargLeft, MargRight);
				}
			}
		}
	}

	return iCustomVerticalAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::CalcAxisLabelSize(TChartAxis* pTCustomAxes)
{
	if(pTCustomAxes->LabelsAngle == 0 || pTCustomAxes->LabelsAngle == 180 || pTCustomAxes->LabelsAngle == 360)
	{
		int iMax = 0;
		for(int i=0; i<pTCustomAxes->Items->Count; ++i)
		{
			int iRows;
			pTCustomAxes->ParentChart->MultiLineTextWidth(pTCustomAxes->Items->Item[i]->Text,iRows);
			if(pTCustomAxes->LabelsAlternate)
			{
				++iRows;
			}
			iMax = Max(iMax, pTCustomAxes->ParentChart->Canvas->FontHeight*iRows);
		}
		return iMax;
	}
	else
	if(pTCustomAxes->LabelsAngle == 90 || pTCustomAxes->LabelsAngle == 270)
	{
		int iMax = 0;
		for(int i=0; i<pTCustomAxes->Items->Count; ++i)
		{
			int tmp;
			iMax = Max(iMax, pTCustomAxes->ParentChart->MultiLineTextWidth(pTCustomAxes->Items->Item[i]->Text,tmp));
		}
		return iMax + 2;
	}
	else
	{
		int iMax = 0;
		for(int i=0; i<pTCustomAxes->Items->Count; ++i)
		{
			int iRows;
			iMax = Max(iMax, pTCustomAxes->ParentChart->MultiLineTextWidth(pTCustomAxes->Items->Item[i]->Text,iRows));
			iMax = Max(iMax, pTCustomAxes->ParentChart->Canvas->FontHeight*iRows);
		}
		return iMax;
	}
}
//---------------------------------------------------------------------------

int CChartTools::CalcVerticalAxisWidth(TChartAxis* pTCustomAxes)
{
	const int extraPos = iDefaultVerticalExtraPos;
	int AxisWidth = 0;

	if(pTCustomAxes)
	{
		AxisWidth = pTCustomAxes->MaxLabelsWidth() + pTCustomAxes->TickLength +
			 CalcVerticalTitleSize(pTCustomAxes->Title) + extraPos;
	}

	return AxisWidth;
}
//---------------------------------------------------------------------------
void CChartTools::StandardVerticalAxes(TCustomChart* Chart, TChartAxis* pTCustomAxes,
	int& NextXLeft, int& NextXRight, int& MargLeft, int& MargRight)
{
	if(pTCustomAxes)
	{
		if(IsAxisVisible(Chart, pTCustomAxes))
		{
			int iAxisWidth = CalcVerticalAxisWidth(pTCustomAxes);
			if(pTCustomAxes->OtherSide)
			{
				pTCustomAxes->PositionPercent = NextXRight;
				NextXRight = NextXRight - iAxisWidth;
			}
			else
			{
				pTCustomAxes->PositionPercent = NextXLeft;
				NextXLeft = NextXLeft - iAxisWidth;
			}
		}
	}
}
//---------------------------------------------------------------------------
bool CChartTools::PlaceVerticalAxes(TCustomChart* Chart, TChartAxis* pTCustomAxes,
	int& NextXLeft, int& NextXRight, int& MargLeft, int& MargRight)
{
	bool bCustomVerticalAxesVisible = false;
	if(pTCustomAxes)
	{
		if(IsAxisVisible(Chart, pTCustomAxes))
		{
			int iAxisWidth = CalcVerticalAxisWidth(pTCustomAxes);
			if(pTCustomAxes->OtherSide)
			{
				pTCustomAxes->PositionPercent = NextXRight;
				NextXRight = NextXRight - iAxisWidth;
				MargRight = MargRight + iAxisWidth;
			}
			else
			{
				pTCustomAxes->PositionPercent = NextXLeft;
				NextXLeft = NextXLeft - iAxisWidth;
				MargLeft = MargLeft + iAxisWidth;
			}
			bCustomVerticalAxesVisible = true;
		}
	}

	return bCustomVerticalAxesVisible;
}
//---------------------------------------------------------------------------
int CChartTools::PlaceBottomHorizontalAxesOnStack(TCustomChart* Chart,
	int& NextXBottom, int& NextXTop, int& MargBottom, int& MargTop)
{
	int iCustomHorizontalAxesCount = 0;

	iCustomHorizontalAxesCount += CountHorizontalBottomCustomAxisDisplayed(Chart);

	if(IsAxisVisible(Chart, Chart->BottomAxis))
	{
		++iCustomHorizontalAxesCount;
	}

	if(iCustomHorizontalAxesCount)
	{
		double dSeparation = 1;
		double dStartPosition = 0.0;
		double dStep = 100.0/iCustomHorizontalAxesCount;
		double dEndPosition = dStep + dSeparation;

		int iMaxHeight = 0;
		int iAxisHeight = 0;

		//przelicz
		if(IsAxisVisible(Chart, Chart->BottomAxis))
		{
			if(Chart->BottomAxis->PositionUnits != muPercent)
			{
				Chart->BottomAxis->PositionUnits = muPercent;
            }
			Chart->BottomAxis->PositionPercent = 0;
			Chart->BottomAxis->StartPosition = dStartPosition;
			Chart->BottomAxis->EndPosition = dEndPosition - dSeparation;

			dStartPosition = Chart->BottomAxis->EndPosition + dSeparation;
			dEndPosition = dStartPosition + dStep;

			iAxisHeight = CalcHorizontalAxisHeight(Chart->BottomAxis);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == true &&
				Chart->CustomAxes->Items[i]->OtherSide == false)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{
					if(Chart->CustomAxes->Items[i]->PositionUnits != muPercent)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPercent;
                    }
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = dStartPosition;
					Chart->CustomAxes->Items[i]->EndPosition = dEndPosition - dSeparation;

					dStartPosition = Chart->CustomAxes->Items[i]->EndPosition + dSeparation;
					dEndPosition = dStartPosition + dStep;

					iMaxHeight = Max(iMaxHeight, CalcHorizontalAxisHeight(Chart->CustomAxes->Items[i]));
				}
			}
		}
		if(iMaxHeight > 0)
		{
			iMaxHeight = iMaxHeight - iAxisHeight;//Jeeli jest o dolna to wymiar jest ju zmniejszony
		}

		NextXBottom = NextXBottom - iMaxHeight;
		MargBottom = MargBottom + iMaxHeight;
	}

	return iCustomHorizontalAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::PlaceTopHorizontalAxesOnStack(TCustomChart* Chart,
	int& NextXBottom, int& NextXTop, int& MargBottom, int& MargTop)
{
	int iCustomHorizontalAxesCount = 0;

	iCustomHorizontalAxesCount += CountHorizontalTopCustomAxisDisplayed(Chart);

	if(IsAxisVisible(Chart, Chart->TopAxis))
	{
		++iCustomHorizontalAxesCount;
	}

	if(iCustomHorizontalAxesCount)
	{
		double dSeparation = 1;
		double dStartPosition = 0.0;
		double dStep = 100.0/iCustomHorizontalAxesCount;
		double dEndPosition = dStep + dSeparation;

		int iMaxHeight = 0;
		int iAxisHeight = 0;

		//przelicz
		if(IsAxisVisible(Chart, Chart->TopAxis))
		{
			if(Chart->TopAxis->PositionUnits != muPercent)
			{
				Chart->TopAxis->PositionUnits = muPercent;
            }
			Chart->TopAxis->PositionPercent = 0;
			Chart->TopAxis->StartPosition = dStartPosition;
			Chart->TopAxis->EndPosition = dEndPosition - dSeparation;

			dStartPosition = Chart->TopAxis->EndPosition + dSeparation;
			dEndPosition = dStartPosition + dStep;

			iAxisHeight = CalcHorizontalAxisHeight(Chart->TopAxis);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == true &&
				Chart->CustomAxes->Items[i]->OtherSide == true)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{
					if(Chart->CustomAxes->Items[i]->PositionUnits != muPercent)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPercent;
                    }
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = dStartPosition;
					Chart->CustomAxes->Items[i]->EndPosition = dEndPosition - dSeparation;

					dStartPosition = Chart->CustomAxes->Items[i]->EndPosition + dSeparation;
					dEndPosition = dStartPosition + dStep;

					iMaxHeight = Max(iMaxHeight, CalcHorizontalAxisHeight(Chart->CustomAxes->Items[i]));
				}
			}
		}
		if(iMaxHeight > 0)
		{
			iMaxHeight = iMaxHeight - iAxisHeight;//Jeeli jest o grna to wymiar jest ju zmniejszony
		}

		NextXTop = NextXTop - iMaxHeight;
		MargTop = MargTop + iMaxHeight;
	}

	return iCustomHorizontalAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::PlaceHorizontalAxes(TCustomChart* Chart,
	int& NextXBottom, int& NextXTop, int& MargBottom, int& MargTop)
{
	int iCustomHorizontalAxesCount = 0;

	if(bHorizontalAxisInOneStack)
	{
		iCustomHorizontalAxesCount += PlaceTopHorizontalAxesOnStack(Chart, NextXBottom, NextXTop, MargBottom, MargTop);
		iCustomHorizontalAxesCount += PlaceBottomHorizontalAxesOnStack(Chart, NextXBottom, NextXTop, MargBottom, MargTop);
	}
	else
	{
		if(IsAxisVisible(Chart, Chart->TopAxis))
		{
			if(Chart->TopAxis->PositionUnits != muPixels)
			{
				Chart->TopAxis->PositionUnits = muPixels;
			}
			Chart->TopAxis->PositionPercent = 0;
			Chart->TopAxis->StartPosition = 0;
			Chart->TopAxis->EndPosition = 100;
			StandardHorizontalAxes(Chart, Chart->TopAxis, NextXBottom, NextXTop, MargBottom, MargTop);
		}

		if(IsAxisVisible(Chart, Chart->BottomAxis))
		{
			if(Chart->BottomAxis->PositionUnits != muPixels)
			{
				Chart->BottomAxis->PositionUnits = muPixels;
			}
			Chart->BottomAxis->PositionPercent = 0;
			Chart->BottomAxis->StartPosition = 0;
			Chart->BottomAxis->EndPosition = 100;
			StandardHorizontalAxes(Chart, Chart->BottomAxis, NextXBottom, NextXTop, MargBottom, MargTop);
		}

		for(int i=0; i < Chart->CustomAxes->Count ;++i)
		{
			if(Chart->CustomAxes->Items[i]->Horizontal == true)
			{
				if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
				{
					if(Chart->CustomAxes->Items[i]->PositionUnits != muPixels)
					{
						Chart->CustomAxes->Items[i]->PositionUnits = muPixels;
                    }
					Chart->CustomAxes->Items[i]->PositionPercent = 0;
					Chart->CustomAxes->Items[i]->StartPosition = 0;
					Chart->CustomAxes->Items[i]->EndPosition = 100;

					iCustomHorizontalAxesCount += PlaceHorizontalAxes(Chart, Chart->CustomAxes->Items[i], NextXBottom, NextXTop, MargBottom, MargTop);
				}
			}
		}
	}

	return iCustomHorizontalAxesCount;
}
//---------------------------------------------------------------------------
int CalcHorizontalTitleSize(TChartAxisTitle* pTChartAxisTitle)
{
	if(pTChartAxisTitle->Visible)
	{
		if(pTChartAxisTitle->Angle == 0 || pTChartAxisTitle->Angle == 180 || pTChartAxisTitle->Angle == 360)
		{
			return pTChartAxisTitle->Height;
		}
		else
		if(pTChartAxisTitle->Angle == 90 || pTChartAxisTitle->Angle == 270)
		{
			return pTChartAxisTitle->Width;
		}
		else
		{
			return Max(pTChartAxisTitle->Width, pTChartAxisTitle->Height);
		}
	}
	else
	{
		return 0;
    }
}
//---------------------------------------------------------------------------
int CalcHorizontalAxisLabelRowsMax(TChartAxis* pTCustomAxes)
{
		int iMaxRows = 0;
		for(int i=0; i<pTCustomAxes->Items->Count; ++i)
		{
			int iRows;
			pTCustomAxes->ParentChart->MultiLineTextWidth(pTCustomAxes->Items->Item[i]->Text,iRows);
			if(pTCustomAxes->LabelsAlternate)
			{
				++iRows;
			}
			iMaxRows = Max(iMaxRows, iRows);
		}
		return iMaxRows;
}
//---------------------------------------------------------------------------
int CChartTools::CalcHorizontalAxisHeight(TChartAxis* pTCustomAxes)
{
	int AxisHeight = 0;
	const int extraPos = iDefaultHorizontalExtraPos;
	if(pTCustomAxes)
	{
		AxisHeight = CalcAxisLabelSize(pTCustomAxes) + pTCustomAxes->TickLength +
			 CalcHorizontalTitleSize(pTCustomAxes->Title) + extraPos;
	}
	return AxisHeight;
}

//---------------------------------------------------------------------------
void CChartTools::StandardHorizontalAxes(TCustomChart* Chart, TChartAxis* pTCustomAxes,
	int& NextXBottom, int& NextXTop, int& MargBottom, int& MargTop)
{
	if(pTCustomAxes)
	{
		if(IsAxisVisible(Chart, pTCustomAxes))
		{
			int	iAxisHeight = CalcHorizontalAxisHeight(pTCustomAxes);
			if(pTCustomAxes->OtherSide)
			{
				pTCustomAxes->PositionPercent = NextXTop;
				NextXTop = NextXTop - iAxisHeight;
			}
			else
			{
				pTCustomAxes->PositionPercent = NextXBottom;
				NextXBottom = NextXBottom - iAxisHeight;
			}
		}
	}
}
//---------------------------------------------------------------------------
bool CChartTools::PlaceHorizontalAxes(TCustomChart* Chart, TChartAxis* pTCustomAxes,
	int& NextXBottom, int& NextXTop, int& MargBottom, int& MargTop)
{
	int bCustomHorizontalAxesCount = false;
	if(pTCustomAxes)
	{
		if(IsAxisVisible(Chart, pTCustomAxes))
		{
			int	iAxisHeight = CalcHorizontalAxisHeight(pTCustomAxes);
			if(pTCustomAxes->OtherSide)
			{
				pTCustomAxes->PositionPercent = NextXTop;
				NextXTop = NextXTop - iAxisHeight;
				MargTop = MargTop + iAxisHeight;
			}
			else
			{
				pTCustomAxes->PositionPercent = NextXBottom;
				NextXBottom = NextXBottom - iAxisHeight;
				MargBottom = MargBottom + iAxisHeight;
			}
			bCustomHorizontalAxesCount = true;
		}
	}

	return bCustomHorizontalAxesCount;
}
//---------------------------------------------------------------------------
TChartSeries* CChartTools::FindFirstActiveSeriesForAxis(TCustomChart* Chart, TChartAxis* pTCustomAxes)
{
	TChartSeries* pTChartSeries = NULL;
	//Os jest widoczna jeeli ma aktywne serie i nie jest ukryta
	if(pTCustomAxes)
	{
		for(int i=0; i<Chart->SeriesCount(); ++i)
		{
			if(Chart->Series[i]->Active &&
				(
					(Chart->Series[i]->HorizAxis == aTopAxis && Chart->TopAxis == pTCustomAxes)
					||
					(Chart->Series[i]->HorizAxis == aBottomAxis && Chart->BottomAxis == pTCustomAxes)
					||
					(Chart->Series[i]->HorizAxis == aBothHorizAxis && Chart->TopAxis == pTCustomAxes && Chart->BottomAxis == pTCustomAxes)
					||
					(Chart->Series[i]->HorizAxis == aCustomHorizAxis && Chart->Series[i]->CustomHorizAxis == pTCustomAxes)
					||
					(Chart->Series[i]->VertAxis == aLeftAxis && Chart->LeftAxis == pTCustomAxes)
					||
					(Chart->Series[i]->VertAxis == aRightAxis && Chart->RightAxis == pTCustomAxes)
					||
					(Chart->Series[i]->VertAxis == aBothVertAxis && Chart->LeftAxis == pTCustomAxes && Chart->RightAxis == pTCustomAxes)
					||
					(Chart->Series[i]->VertAxis == aCustomVertAxis && Chart->Series[i]->CustomVertAxis == pTCustomAxes)
				)
			)
			{
				//poszukiwanie aktywnej serii zwizanej z osi
				pTChartSeries = Chart->Series[i];
				break;
			}
		}
	}

	return pTChartSeries;
}
//---------------------------------------------------------------------------
bool CChartTools::IsAxisVisible(TCustomChart* Chart, TChartAxis* pTCustomAxes)
{
	return pTCustomAxes->Visible && (Chart->CountActiveSeries() == 0 || FindFirstActiveSeriesForAxis(Chart, pTCustomAxes)!= NULL);
}
//---------------------------------------------------------------------------
bool CChartTools::IsLegendVisible(TChartLegend* Legend)
{
	bool bActiveSeries = true;
	TCustomChart* pTCustomChart = dynamic_cast<TCustomChart*>(Legend->ParentChart);
	if(pTCustomChart)
	{
		bActiveSeries = pTCustomChart->CountActiveSeries() > 0;
	}
	return (Legend->Visible && Legend->Height > 0 && Legend->Width > 0 &&
		Legend->CustomPosition && Legend->TotalLegendItems > 0 && bActiveSeries);
}
//---------------------------------------------------------------------------
int CChartTools::CountHorizontalTopCustomAxisDisplayed(TCustomChart* Chart)
{
	int iCustomAxesCount = 0;
	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == true &&
			Chart->CustomAxes->Items[i]->OtherSide == true)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomAxesCount;
			}
		}
	}
	return iCustomAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::CountHorizontalBottomCustomAxisDisplayed(TCustomChart* Chart)
{
	int iCustomAxesCount = 0;
	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == true &&
			Chart->CustomAxes->Items[i]->OtherSide == false)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomAxesCount;
			}
		}
	}
	return iCustomAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::CountVerticalLeftCustomAxisDisplayed(TCustomChart* Chart)
{
	int iCustomAxesCount = 0;
	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == false &&
			Chart->CustomAxes->Items[i]->OtherSide == false)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomAxesCount;
			}
		}
	}
	return iCustomAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::CountVerticalRightCustomAxisDisplayed(TCustomChart* Chart)
{
	int iCustomAxesCount = 0;
	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(Chart->CustomAxes->Items[i]->Horizontal == false &&
			Chart->CustomAxes->Items[i]->OtherSide == true)
		{
			if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
			{
				++iCustomAxesCount;
			}
		}
	}
	return iCustomAxesCount;
}
//---------------------------------------------------------------------------
int CChartTools::CountAllCustomAxisDisplayed(TCustomChart* Chart)
{
	int iCustomAxesCount = 0;
	for(int i=0; i < Chart->CustomAxes->Count ;++i)
	{
		if(IsAxisVisible(Chart, Chart->CustomAxes->Items[i]))
		{
			++iCustomAxesCount;
		}
	}
	return iCustomAxesCount;
}
//---------------------------------------------------------------------------
void CChartTools::PlaceChartObjects(TCustomChart* Chart)
{
	int iLegendMargin = iDefaultLegendMargin;

	int NextXLeft = 0;
	int NextXRight = 0;
	int MargLeft = iDefaultMargLeft;
	int MargRight = iDefaultMargRight;

	int iChartBoundsWidth = Chart->ChartBounds.Width();
	int iChartBoundsHeight = Chart->ChartBounds.Height();

	int iCountHorizontalTopCustomAxisDisplayed = CountHorizontalTopCustomAxisDisplayed(Chart);
	int iCountHorizontalBottomCustomAxisDisplayed = CountHorizontalBottomCustomAxisDisplayed(Chart);

	int iCountVerticalLeftCustomAxisDisplayed = CountVerticalLeftCustomAxisDisplayed(Chart);
	int iCountVerticalRightCustomAxisDisplayed = CountVerticalRightCustomAxisDisplayed(Chart);

    if(Chart->MarginUnits != muPixels)
	{
		//Wszystkie obliczenia wykonywane s na Piexelach
		Chart->MarginUnits = muPixels;
	}
	if(Chart->Legend->PositionUnits != muPixels)
	{
		Chart->Legend->PositionUnits = muPixels;
	}
	if(Chart->Legend->CustomPosition != true)
	{
		Chart->Legend->CustomPosition = true;
	}
	if(Chart->Title->CustomPosition != true)
	{
		Chart->Title->CustomPosition = true;
    }
	if(Chart->SubTitle->CustomPosition != true)
	{
		Chart->SubTitle->CustomPosition = true;
	}
	if(Chart->Foot->CustomPosition != true)
	{
		Chart->Foot->CustomPosition = true;
	}
	if(Chart->SubFoot->CustomPosition != true)
	{
		Chart->SubFoot->CustomPosition = true;
	}

	/*Vertical*/
	if(Chart->Legend->Alignment == laLeft)
	{
		if(IsLegendVisible(Chart->Legend))
		{
			int iLegendWidth = Chart->Legend->Width;
			MargLeft = MargLeft + iLegendWidth + 2*iLegendMargin;
			Chart->Legend->Top = (iChartBoundsHeight - Chart->Legend->Height)/2;
			Chart->Legend->Left = Chart->Left + iLegendMargin;
		}
	}

	bool bCustomVerticalAxes = PlaceVerticalAxes(Chart, NextXLeft, NextXRight, MargLeft, MargRight);

	if(Chart->Legend->Alignment == laRight)
	{
		if(IsLegendVisible(Chart->Legend))
		{
			int iLegendWidth = Chart->Legend->Width;
			MargRight = MargRight + iLegendWidth + 2*iLegendMargin;
			Chart->Legend->Top = (iChartBoundsHeight - Chart->Legend->Height)/2;
			Chart->Legend->Left = Chart->Left + iChartBoundsWidth - iLegendWidth - iLegendMargin;
		}
	}

	/*Horizontal*/
	int NextXBottom = 0;
	int NextXTop = 0;
	int MargBottom = iDefaultMargBottom;
	int MargTop = iDefaultMargTop;

	//Uwzgldnij miejsce na tytu
	if(Chart->Title->Visible && Chart->Title->Height > 0)
	{
		int iTitleHeight = Chart->Title->Height + Chart->Title->VertMargin
				+ (Chart->Title->Shadow->Visible ? Chart->Title->Shadow->VertSize : 0)
				+ (Chart->Title->Emboss->Visible ? Chart->Title->Emboss->VertSize : 0);

		Chart->Title->Top = MargTop;
		Chart->Title->Left = (iChartBoundsWidth - Chart->Title->Width)/2;
		MargTop = MargTop + iTitleHeight;
	}
	//Uwzgldnij miejsce na potytu
	if(Chart->SubTitle->Visible && Chart->SubTitle->Height > 0)
	{
		int iSubTitleHeight = Chart->SubTitle->Height + Chart->SubTitle->VertMargin
				+ (Chart->SubTitle->Shadow->Visible ? Chart->SubTitle->Shadow->VertSize : 0)
				+ (Chart->SubTitle->Emboss->Visible ? Chart->SubTitle->Emboss->VertSize : 0);

		Chart->SubTitle->Top = MargTop;
		Chart->SubTitle->Left = (iChartBoundsWidth - Chart->SubTitle->Width)/2;
		MargTop = MargTop + iSubTitleHeight;
	}

	if(Chart->Legend->Alignment == laTop)
	{
		if(IsLegendVisible(Chart->Legend))
		{
			int iLegendHeight = Chart->Legend->Height + Chart->Legend->VertMargin
				+ (Chart->Legend->Shadow->Visible ? Chart->Legend->Shadow->VertSize : 0)
				+ (Chart->Legend->Emboss->Visible ? Chart->Legend->Emboss->VertSize : 0);

			Chart->Legend->Top = MargTop;
			Chart->Legend->Left = (iChartBoundsWidth - Chart->Legend->Width)/2;
			MargTop = MargTop + iLegendHeight;
		}
	}

	bool bCustomHorizontalAxes = PlaceHorizontalAxes(Chart, NextXBottom, NextXTop, MargBottom, MargTop);

	//Uwzgldnij miejsce na stopk
	if(Chart->Foot->Visible && Chart->Foot->Height > 0)
	{
		int iFootHeight = Chart->Foot->Height + Chart->Foot->VertMargin
				+ (Chart->Foot->Shadow->Visible ? Chart->Foot->Shadow->VertSize : 0)
				+ (Chart->Foot->Emboss->Visible ? Chart->Foot->Emboss->VertSize : 0);

		Chart->Foot->Top = MargTop + Chart->ChartHeight + MargBottom;
		Chart->Foot->Left = (iChartBoundsWidth - Chart->Foot->Width)/2;
		MargBottom = MargBottom + iFootHeight;
	}

	//Uwzgldnij miejsce na podstopk
	if(Chart->SubFoot->Visible && Chart->SubFoot->Height > 0)
	{
		int iSubFootHeight = Chart->SubFoot->Height + Chart->SubFoot->VertMargin
				+ (Chart->SubFoot->Shadow->Visible ? Chart->SubFoot->Shadow->VertSize : 0)
				+ (Chart->SubFoot->Emboss->Visible ? Chart->SubFoot->Emboss->VertSize : 0);

		Chart->SubFoot->Top = MargTop + Chart->ChartHeight + MargBottom;
		Chart->SubFoot->Left = (iChartBoundsWidth - Chart->SubFoot->Width)/2;
		MargBottom = MargBottom + iSubFootHeight;
	}

	if(Chart->Legend->Alignment == laBottom)
	{
		if(IsLegendVisible(Chart->Legend))
		{
			int iLegendHeight = Chart->Legend->Height + Chart->Legend->VertMargin
				+ (Chart->Legend->Shadow->Visible ? Chart->Legend->Shadow->VertSize : 0)
				+ (Chart->Legend->Emboss->Visible ? Chart->Legend->Emboss->VertSize : 0);

			Chart->Legend->Top = Chart->ChartBounds.Height() - iLegendHeight;

			Chart->Legend->Left = (iChartBoundsWidth - Chart->Legend->Width)/2;
			MargBottom = MargBottom + iLegendHeight;
		}
	}

	if(Chart->MarginTop != MargTop)
	{
		Chart->MarginTop = MargTop;
	}
	if(Chart->MarginBottom != MargBottom)
	{
		Chart->MarginBottom = MargBottom;
	}
	if(Chart->MarginLeft != MargLeft)
	{
		Chart->MarginLeft = MargLeft;
	}
	if(Chart->MarginRight != MargRight)
	{
		Chart->MarginRight = MargRight;
	}
}
//---------------------------------------------------------------------------
void CChartTools::CustomHorizontalAxisZoom(TChartAxis* pTCustomHorizontalAxes)
{
	double aMin = pTCustomHorizontalAxes->CalcPosPoint(pTCustomHorizontalAxes->ParentChart->Zoom->X0);
	double aMax = pTCustomHorizontalAxes->CalcPosPoint(pTCustomHorizontalAxes->ParentChart->Zoom->X1);
	pTCustomHorizontalAxes->Automatic = false;
	pTCustomHorizontalAxes->SetMinMax(aMin,aMax);
}
//---------------------------------------------------------------------------
void CChartTools::CustomVerticalAxisZoom(TChartAxis* pTCustomVerticalAxes)
{
	double aMin = pTCustomVerticalAxes->CalcPosPoint(pTCustomVerticalAxes->ParentChart->Zoom->Y0);
	double aMax = pTCustomVerticalAxes->CalcPosPoint(pTCustomVerticalAxes->ParentChart->Zoom->Y1);
	pTCustomVerticalAxes->Automatic = false;
	pTCustomVerticalAxes->SetMinMax(aMin,aMax);
}
//---------------------------------------------------------------------------
void CChartTools::CustomAxisZoom(TCustomChart* Chart)
{
	if(Chart->Zoomed)
	{
		for(int i=0; i<Chart->SeriesCount(); ++i)
		{
			if(Chart->Zoom->Direction == tzdBoth || Chart->Zoom->Direction == tzdHorizontal)
			{
				if(Chart->Series[i]->HorizAxis == aCustomHorizAxis)
				{
					TChartAxis* pTCustomHorizontalAxes = Chart->Series[i]->GetHorizAxis;
					if(pTCustomHorizontalAxes)
					{
						CustomHorizontalAxisZoom(pTCustomHorizontalAxes);
					}
				}
			}

			if(Chart->Zoom->Direction == tzdBoth || Chart->Zoom->Direction == tzdVertical)
			{
				if(Chart->Series[i]->VertAxis == aCustomVertAxis)
				{
					TChartAxis* pTCustomVerticalAxes = Chart->Series[i]->GetVertAxis;
					if(pTCustomVerticalAxes)
					{
						CustomVerticalAxisZoom(pTCustomVerticalAxes);
					}
				}
			}
		}
	}
}
//---------------------------------------------------------------------------
void CChartTools::CustomAxisUndoZoom(TCustomChart* Chart, bool bAutomatic)
{
	if(!Chart->Zoomed)
	{
		for(int i=0; i<Chart->SeriesCount(); ++i)
		{
			if(Chart->Zoom->Direction == tzdBoth || Chart->Zoom->Direction == tzdHorizontal)
			{
				TChartAxis* pTCustomHorizontalAxes = Chart->Series[i]->GetHorizAxis;
				if(pTCustomHorizontalAxes)
				{
					pTCustomHorizontalAxes->Automatic = true;
					pTCustomHorizontalAxes->AdjustMaxMin();
					pTCustomHorizontalAxes->Automatic = bAutomatic;

				}
			}

			if(Chart->Zoom->Direction == tzdBoth || Chart->Zoom->Direction == tzdVertical)
			{
				TChartAxis* pTCustomVerticalAxes = Chart->Series[i]->GetVertAxis;
				if(pTCustomVerticalAxes)
				{
					pTCustomVerticalAxes->Automatic = true;
					pTCustomVerticalAxes->AdjustMaxMin();
					pTCustomVerticalAxes->Automatic = bAutomatic;
				}
			}
		}
	}
}
//---------------------------------------------------------------------------
bool CChartTools::CalcBarPenVisibility(TCustomChart* pTChart)
{
	bool bResults = false;//true - tylko jeeli nastpia zmiana
	for(int i=0; i<pTChart->SeriesCount();++i)
	{
		TBarSeries* pTBarSeries = dynamic_cast<TBarSeries*>(pTChart->Series[i]);
		if(pTBarSeries)
		{
			if(pTBarSeries->BarBounds.Width() > 4*pTBarSeries->BarPen->Width)
			{
				if(pTBarSeries->BarPen->Visible == false)
				{
					pTBarSeries->BarPen->Visible = true;
					bResults = true;
				}
			}
			else
			{
				if(pTBarSeries->BarPen->Visible == true)
				{
					pTBarSeries->BarPen->Visible = false;
					bResults = true;
				}
			}
		}
		else
		{
			TAreaSeries* pTAreaSeries = dynamic_cast<TAreaSeries*>(pTChart->Series[i]);
			if(pTAreaSeries)
			{
				if(pTAreaSeries->XValues->Count)
				{
					double dRange = pTAreaSeries->XValues->Range();
					double dCalcXSizeValue = pTAreaSeries->CalcXSizeValue(dRange)/pTAreaSeries->XValues->Count;

					if(dCalcXSizeValue > 4*pTAreaSeries->AreaLinesPen->Width)
					{
						if(pTAreaSeries->AreaLinesPen->Visible == false)
						{
							pTAreaSeries->AreaLinesPen->Visible = true;
							bResults = true;
						}
					}
					else
					{
						if(pTAreaSeries->AreaLinesPen->Visible == true)
						{
							pTAreaSeries->AreaLinesPen->Visible = false;
							bResults = true;
						}
					}
				}
			}
		}
	}

	return bResults;
}
//---------------------------------------------------------------------------
void CChartTools::PrepareChartAxisColor(TChartAxis* pTChartAxis)
{
	if(pTChartAxis)
	{
		for(int i=0; i<pTChartAxis->ParentChart->SeriesCount(); ++i)
		{
			if((pTChartAxis->ParentChart->Series[i]->CustomVertAxis == pTChartAxis ||
				pTChartAxis->ParentChart->Series[i]->CustomHorizAxis == pTChartAxis)&&
				pTChartAxis->ParentChart->Series[i]->Active)
			{
				//Kolor ostatniej znalezionej serii
				pTChartAxis->Axis->Color = pTChartAxis->ParentChart->Series[i]->Color;
				pTChartAxis->Title->Pen->Color = pTChartAxis->Axis->Color;
				pTChartAxis->Title->Shadow->Color = pTChartAxis->Axis->Color;
			}
		}
	}
}
//---------------------------------------------------------------------------
void CChartTools::PrepareChartAxis(TChartAxis* pTChartAxis)
{
	if(pTChartAxis)
	{
		PrepareChartAxisColor(pTChartAxis);

		pTChartAxis->Grid->Visible = false;
		pTChartAxis->PositionUnits = muPixels;
		pTChartAxis->Title->Color = clWhite;
		pTChartAxis->Title->Transparent = false;
		pTChartAxis->Automatic = true;

		if(pTChartAxis->Horizontal == true)
		{
			pTChartAxis->ExactDateTime = true;
			pTChartAxis->Increment = DateTimeStep[dtFifteenMinutes];
			//pTChartAxis->DateTimeFormat = "c"; //Do czasu naprawienia bdu w Chart-ach http://bugs.teechart.net/show_bug.cgi?id=1692
			pTChartAxis->DateTimeFormat = "yyyy-mm-dd hh:mm";
			pTChartAxis->LabelsMultiLine = true;
		}
	}
}
//---------------------------------------------------------------------------
bool CChartTools::IsCustomAxis(TChartAxis* pTChartAxis)
{
	bool bResults = false;
	if(pTChartAxis)
	{
		for(int i=0; i<pTChartAxis->ParentChart->CustomAxes->Count; ++i)
		{
			if(pTChartAxis->ParentChart->CustomAxes->Items[i] == pTChartAxis)
			{
				bResults = true;
				break;
			}
		}
	}
	return bResults;
}
//---------------------------------------------------------------------------
void CChartTools::PrepareAxisVisibility(TCustomChart* pTChart)
{
	//Wycz osie dla krych nie ma aktywnych serii
	if(pTChart)
	{
		pTChart->LeftAxis->Visible = (FindFirstActiveSeriesForAxis(pTChart, pTChart->LeftAxis) != NULL);
		pTChart->RightAxis->Visible = (FindFirstActiveSeriesForAxis(pTChart, pTChart->RightAxis) != NULL);
		pTChart->BottomAxis->Visible = (FindFirstActiveSeriesForAxis(pTChart, pTChart->BottomAxis) != NULL);
		pTChart->TopAxis->Visible = (FindFirstActiveSeriesForAxis(pTChart, pTChart->TopAxis) != NULL);

		for(int i=0; i<pTChart->CustomAxes->Count; ++i)
		{
			pTChart->CustomAxes->Items[i]->Visible = (FindFirstActiveSeriesForAxis(pTChart, pTChart->CustomAxes->Items[i]) != NULL);
		}
	}
}
//---------------------------------------------------------------------------
TChartAxis* CChartTools::FindFirstVisibleAxis(TCustomChart* Chart, bool bHorizontal)
{
	for(int i=0; i<Chart->Axes->Count; ++i)
	{
		if(Chart->Axes->Items[i]->Horizontal == bHorizontal)
		{
			if(FindFirstActiveSeriesForAxis(Chart, Chart->Axes->Items[i]) != NULL)
			{
				return Chart->Axes->Items[i];
			}
		}
	}
	return NULL;
}
//---------------------------------------------------------------------------

